%% Matrix exponential solution
% This code provides a closed form solution for linear compartmental models. 
%
% This code was developed by Stephen Duffull
% October 2014
% Otago Pharmacometrics Group, University of Otago
%
% Note optimisation of the code was not performed here in an attempt to
% make it as accessible as possible.
%

%% Example
% An example to simplify a 3-cpt PK model of midazolam into a 1 or 2-cpt PK model (iv bolus) 
% Model details from: Brill et al, Clin Pharmacokinet 2014 (doi:10.1007/s40262-014-0166-x)

%% Outline of code
% Coding below covers 5 parts: 
% 1)Initialising PK model
% 2)matrix exponential solution for the original model  
% 3)output plot

%% PART 1: simulation setup for original PK model 

dose = 5;  % unit: mg, iv bolus
TT = [0:2:360];  % unit: min, 0-360 mins with a time step of every 2 mins  

%%% a vector of initial conditions
A0=[dose; 
    0
    0];  

% original parameter values
% Brill et al reported clearance & volume, which have been converted to rate constants
k10 = 0.010;     % unit: per min
k12 = 0.036;     % unit: per min
k13 = 0.0042;    % unit: per min
k21 = 0.0171;    % unit: per min
k31 = 0.002;     % unit: per min 
V1  = 36.4;      % central volume, unit: L

% rate constant matrix in original model
K = [-(k12+k13+k10) k21     k31
        k12         -k21    0
        k13         0       -k31];
    
%% PART 2: matrix exponential solution for the original model 

% matrix exponential solution requires eigenvalues and eigenvectors of rate constant matrix in original model
[P_m,Lam_m]=eig(K);

% eigenvalue/eigenvector decomposition for each state individually
q_m=inv(P_m)*A0;
R_m = zeros(length(q_m),length(TT));
for j_m=1:length(q_m)
    R_m(j_m,:)=exp(Lam_m(j_m,j_m)*TT)*q_m(j_m);
end

% obtain matrix of amounts in all cpts in original model: row for cpt number, column for time point
A = P_m*R_m; 

%% PART 3: output plots %%

% plot central concentration vs. time in both models 
% in original model
plot(TT,A(1,:)/V1,'b-','LineWidth',2) 

xlabel('Time (min)','fontsize',14);
ylabel('Concentration (mg/L)','fontsize',14);


